/* This file is part of the Kiretu, a Kinect reconstruction tutor.
 * 
 * http://pille.iwr.uni-heidelberg.de/~kinect01/
 * 
 * The code ist licensed to you under the terms of the GNU General Public 
 * License, version 2.0. See the GPL2 file for the text of the license or the 
 * following URL:
 * 
 * http://www.gnu.org/licenses/gpl-2.0.txt
 * 
 * If you redistribute this file in source form, modified or unmodified, you
 * have to leave this header intact and distribute it under the same terms with 
 * the GPL2 file.
 * 
 * Binary distributions must follow the binary distribution requirements of
 * the License.
 * 
 */
 
 #include "Hud.h"

Hud::Hud() {
}

Hud::~Hud() {

}

void Hud::printGLText(std::string text, bool printLarge) {
	
	for (int i = 0; i < text.length(); i++) {
		
		if (printLarge) {
			glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,text[i]);
		} else {
			glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,text[i]);
		}
	}
}

void Hud::printGLTextPos(int x, int y, std::string text, bool printLarge) {
	glRasterPos2i(x,y);
	printGLText(text, printLarge);
}

void Hud::printGLLabel()  {
		
	glColor4f(0,0,0,0.4);
	glBegin(GL_QUADS);
		glVertex3i(0,0,0);
		glVertex3i(0,30,0);
		glVertex3i(100,30,0);
		glVertex3i(100,0,0);
		
		glVertex3i(640,0,0);
		glVertex3i(640,30,0);
		glVertex3i(765,30,0);
		glVertex3i(765,0,0);
	glEnd();
	
	glColor4f(1,1,1,1);
	printGLTextPos(5, 20, "Depth Map", true);
	printGLTextPos(645, 20, "Video Output", true);
}

void Hud::printGLScanning(bool print) {
		
	if (print) {
		
		int x[2] = {275, 226};	// north-west corner of box
		int y[2] = {365, 257};	// south-east corner of box 
		
		glColor4f(0,0,0,0.5);
		glRecti(x[0], x[1], y[0], y[1]);
		
		glColor4f(1, 1, 1, 1);
		printGLTextPos(281, 248, "Scanning", true);
	}	
}

void Hud::printGLHelp(bool print) {
	
	int x[2];	// north-west corner of box
	int y[2];	// south-east corner of box
	
	if (print) {
		
		// north-west corner of box
		x[0] = 0;
		x[1] = 247;
		
		// south-east corner of box
		y[0] = 180;
		y[1] = 480;
		
		int textPos[2] = {5, 380};
		int skip = 18;
		int smallSkip = 15;
		int indent = 10;
		
		glColor4f(0,0,0,0.5);
		glRecti(x[0], x[1], y[0], y[1]);
		
		// Kinect-Status
		
		textPos[1] = 270;
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Help", true);
		
		textPos[1] = 290;
		
		printGLTextPos(textPos[0], textPos[1], "Change Video Format", false);
		textPos[1] += smallSkip;
		printGLTextPos(textPos[0] + indent, textPos[1], "F", false);
		textPos[1] += skip;
		
		printGLTextPos(textPos[0], textPos[1], "Move Kinect Up/Center/Down", false);
		textPos[1] += smallSkip;
		printGLTextPos(textPos[0] + indent, textPos[1], "W/S/X", false);
		textPos[1] += skip;
		
		printGLTextPos(textPos[0], textPos[1], "Start Scan", false);
		textPos[1] += smallSkip;
		printGLTextPos(textPos[0] + indent, textPos[1], "Spacebar", false);
		textPos[1] += skip;
		
		printGLTextPos(textPos[0], textPos[1], "Toggle Reconstruction-Steps", false);
		textPos[1] += smallSkip;
		printGLTextPos(textPos[0] + indent, textPos[1], "M/D/E/R (see Info)", false);
		textPos[1] += skip;
		
		printGLTextPos(textPos[0], textPos[1], "Toggle Help/Info", false);
		textPos[1] += smallSkip;
		printGLTextPos(textPos[0] + indent, textPos[1], "H/I", false);
		textPos[1] += skip;
		
		printGLTextPos(textPos[0], textPos[1], "Exit", false);
		textPos[1] += smallSkip;
		printGLTextPos(textPos[0] + indent, textPos[1], "Esc", false);
		textPos[1] += skip;
		
		
	} else {
		
		// north-west corner of box
		x[0] = 0;
		x[1] = 457;
		
		// south-east corner of box
		y[0] = 103;
		y[1] = 480;
		
		int textPos[2] = {5, 473};
		
		glColor4f(0,0,0,0.5);
		glRecti(x[0], x[1], y[0], y[1]);
		
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Press H for Help", false);
	}
}

void Hud::printGLState(bool print, int angle, std::string format, bool states[5]) {
	
	int x[2];	// north-west corner of box
	int y[2];	// south-east corner of box
	
	if (print) {
		
		// north-west corner of box
		x[0] = 1110;
		x[1] = 300;
		
		// south-east corner of box
		y[0] = 1280;
		y[1] = 480;
		
		int textPos[2] = {1120, 380};
		int skip = 15;
		
		glColor4f(0,0,0,0.5);
		glRecti(x[0], x[1], y[0], y[1]);
		
		// Kinect-Status
		
		textPos[1] = 330;
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Kinect-Status", true);
		
		textPos[1] = 350;
		
		printGLTextPos(textPos[0], textPos[1], "Format: ", false);
		printGLTextPos(textPos[0] + 50, textPos[1], format, false);
		
		textPos[1] += skip;
		
		printGLTextPos(textPos[0], textPos[1], "Angle: ", false);
		printGLTextPos(textPos[0] + 50, textPos[1], intToStr(angle), false);
		
		
		
		// Reconstruction-Steps
		
		textPos[1] = 390;
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Reconstruction", true);
		
		textPos[0] = 1130;
		textPos[1] = 410;
		
		
		// Raw-to-Meter
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Raw to Meter (M)", false);
		
		if (states[0]) {
			glColor4f(0,1,0,1);
		} else {
			glColor4f(1,0,0,1);
		}
		
		glRecti(textPos[0]-10, textPos[1]-7, textPos[0]-5, textPos[1]-2);	
		
		textPos[1] += skip;
		
		
		// Depth-to-Cloud
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Depth to Cloud (D)", false);
		if (states[1]) {
			glColor4f(0,1,0,1);
		} else {
			glColor4f(1,0,0,1);
		}
		
		glRecti(textPos[0]-10, textPos[1]-7, textPos[0]-5, textPos[1]-2);
		
		textPos[1] += skip;
		
		
		// Extrinsics
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Apply Extrinsics (E)", false);
		if (states[2]) {
			glColor4f(0,1,0,1);
		} else {
			glColor4f(1,0,0,1);
		}
		
		glRecti(textPos[0]-10, textPos[1]-7, textPos[0]-5, textPos[1]-2);
		
		textPos[1] += skip;
		
		
		// RGB-Mapping
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Cloud-RGB-Mapping (R)", false);
		if (states[3]) {
			glColor4f(0,1,0,1);
		} else {
			glColor4f(1,0,0,1);
		}
		
		glRecti(textPos[0]-10, textPos[1]-7, textPos[0]-5, textPos[1]-2);
		
		textPos[1] += skip;
		
		// Normals
		//~ glColor4f(1, 1, 1, 1);
		//~ printGLTextPos(textPos[0], textPos[1], "Compute Normals (N)", false);
		//~ if (states[4]) {
			//~ glColor4f(0,1,0,1);
		//~ } else {
			//~ glColor4f(1,0,0,1);
		//~ }
		//~ 
		//~ glRecti(textPos[0]-10, textPos[1]-7, textPos[0]-5, textPos[1]-2);
	
	} else {
		
		// north-west corner of box
		x[0] = 1185;
		x[1] = 457;
		
		// south-east corner of box
		y[0] = 1280;
		y[1] = 480;
		
		int textPos[2] = {1192, 473};
		
		glColor4f(0,0,0,0.5);
		glRecti(x[0], x[1], y[0], y[1]);
		
		glColor4f(1, 1, 1, 1);
		printGLTextPos(textPos[0], textPos[1], "Press I for Info", false);
	}
}
