/* This file is part of the Kiretu, a Kinect reconstruction tutor.
 * 
 * http://pille.iwr.uni-heidelberg.de/~kinect01/
 * 
 * The code ist licensed to you under the terms of the GNU General Public 
 * License, version 2.0. See the GPL2 file for the text of the license or the 
 * following URL:
 * 
 * http://www.gnu.org/licenses/gpl-2.0.txt
 * 
 * If you redistribute this file in source form, modified or unmodified, you
 * have to leave this header intact and distribute it under the same terms with 
 * the GPL2 file.
 * 
 * Binary distributions must follow the binary distribution requirements of
 * the License.
 * 
 */
 
 /** \file KinectCloud.cpp
 * 
 * Implementation of the KinectCloud-class. See header-file for description.
 * 
 * \author		Daniel Wunderlich (d.wunderlich@stud.uni-heidelberg.de)
 * \version		0.5
 * \date		2011-01-08
 */

#include "KinectCloud.h"

KinectCloud::KinectCloud() :
	cloud(640*480, std::vector<float>(3)),
	cloudRgb(640*480, std::vector<int>(3)),
	cloudRgbMapping(640*480, std::vector<int>(2)),
	//~ normals(640*480, std::vector<float>(3)),
	validValues(640*480),
	reconstructionSteps(5) {
	
	
	// Initialize default RGB-depth-mapping
	std::vector<int> temp(2);
	
	for (int i = 0; i < 480; i++) {
		for (int j = 0; j < 640; j++) {
			
			temp[0] = j;
			temp[1] = i;
			cloudRgbMapping[i*640 + j] = temp;
		}
	}
	
	// Initialize reconstruction-steps
	reconstructionSteps[0] = false;
	reconstructionSteps[1] = false;
	reconstructionSteps[2] = false;
	reconstructionSteps[3] = false;
}

KinectCloud::~KinectCloud() {
	
}

void KinectCloud::addZValues(int* zValues) {
	
	std::vector<float> point(3);
	
	for (int i = 0; i < 480; i++) {
		for (int j = 0; j < 640; j++) {
		
			if (zValues[i*640 + j] > 0) {
				point[0] = j;
				point[1] = i;
				point[2] = zValues[i*640 + j];
				
				cloud[i*640 + j] = point;
				
				validValues[i*640 + j] = true;
				
			} else {
				
				point[0] = 0;
				point[1] = 0;
				point[2] = 0;
				
				cloud[i*640 + j] = point;
				
				validValues[i*640 + j] = false;
			}
		}
	}
}

void KinectCloud::addRgbValues(int* rgbValues) {
	
	std::vector<int> point(3);
	
	for (int i = 0; i < 640*480; i++) {
		point[0] = rgbValues[i*3 + 0];
		point[1] = rgbValues[i*3 + 1];
		point[2] = rgbValues[i*3 + 2];
		
		cloudRgb[i] = point;
	}
}

//~ void KinectCloud::undistortDepthImage(std::vector<float> depthDist) {
	//~ 
	//~ float k1 = depthDist[0];
	//~ float k2 = depthDist[1];
	//~ float p1 = depthDist[2];
	//~ float p2 = depthDist[3];
	//~ float k3 = depthDist[4];
	//~ 
	//~ std::cout << "Dist: " << k1 << " " << k2 << " " << p1 << " " << p2 << " " << k3 << std::endl;
	//~ 
	//~ float r;
	//~ float xNew;
	//~ float yNew;
	//~ 
	//~ for (int i = 0; i < cloud.size(); i++) {
		//~ 
		//~ r = sqrt(pow(cloud[i][0], 2) + pow(cloud[i][1], 2));
		//~ 
		//~ xNew = cloud[i][0] * (1 + k1*pow(r, 2) + k2*pow(r, 4) + k3*pow(r, 6)) + 2*p1*cloud[i][0]*cloud[i][1] + p2*(pow(r, 2) + 2*pow(cloud[i][0], 2));
		//~ yNew = cloud[i][1] * (1 + k1*pow(r, 2) + k2*pow(r, 4) + k3*pow(r, 6)) + 2*p2*cloud[i][0]*cloud[i][1] + p1*(pow(r, 2) + 2*pow(cloud[i][1], 2));
		//~ 
		//~ cloud[i][0] = xNew;
		//~ cloud[i][1] = yNew;
	//~ }
	//~ 
//~ }

//~ void KinectCloud::undistortRgbImage(std::vector<float> rgbDist) {
	//~ 
	//~ float k1 = rgbDist[0];
	//~ float k2 = rgbDist[1];
	//~ float p1 = rgbDist[2];
	//~ float p2 = rgbDist[3];
	//~ float k3 = rgbDist[4];
	//~ 
	//~ float r;
	//~ 
	//~ for (int i = 0; i < cloudRgb.size(); i++) {
		//~ 
		//~ r = sqrt(pow(cloudRgb[i][0], 2) + pow(cloudRgb[i][1]));
		//~ 
		//~ cloud[i][0] = cloud[i][0] * (1 + k1*pow(r, 2) + k2*pow(r, 4) + k3*pow(r, 6)) + 2*p1*cloud[i][0]*cloud[i][1] + p2*(pow(r, 2) + 2*pow(cloud[i][0], 2));
		//~ cloud[i][1] = cloud[i][1] * (1 + k1*pow(r, 2) + k2*pow(r, 4) + k3*pow(r, 6)) + 2*p2*cloud[i][0]*cloud[i][1] + p1*(pow(r, 2) + 2*pow(cloud[i][1], 2));
	//~ }
//~ }

void KinectCloud::rawToMeter() {
	
	for (int i = 0; i < cloud.size(); i++) {
		cloud[i][2] = 1.0/(cloud[i][2] * -0.0030711016 + 3.3309495161);
	}
	
	reconstructionSteps[0] = true;
}

void KinectCloud::depthToCloud(std::vector< std::vector<float> > depthCam) {
	
	float fx = depthCam[0][0];
	float fy = depthCam[1][1];
	float cx = depthCam[0][2];
	float cy = depthCam[1][2];
	
	for (int i = 0; i < cloud.size(); i++) {
		cloud[i][0] = (cloud[i][0] - cx) * cloud[i][2] / fx;
		cloud[i][1] = (cloud[i][1] - cy) * cloud[i][2] / fy;
	}
	
	reconstructionSteps[1] = true;
}
	
void KinectCloud::applyExtrinsics(std::vector< std::vector<float> > rot, std::vector<float> trans) {
	
	std::vector< std::vector<float> > tempCloud = cloud;
	
	for (int i = 0; i < cloud.size(); i++) {
		matMult(rot, tempCloud[i], cloud[i]);
	}
	
	tempCloud = cloud;
	
	for (int i = 0; i < cloud.size(); i++) {
		vecAdd(trans, tempCloud[i], cloud[i]);
	}
	
	reconstructionSteps[2] = true;
}

void KinectCloud::computeRgbMapping(std::vector< std::vector<float> > rgbCam) {
	
	float fx = rgbCam[0][0];
	float fy = rgbCam[1][1];
	float cx = rgbCam[0][2];
	float cy = rgbCam[1][2];
	
	std::vector<int> px(2);
	
	for (int i = 0; i < 640*480; i++) {
			
			px[0] = (int) (cloud[i][0] * fx / cloud[i][2]) + cx;
			px[1] = (int) (cloud[i][1] * fy / cloud[i][2]) + cy;
			
			cloudRgbMapping[i] = px;
	}
	
	reconstructionSteps[3] = true;
}

//~ void KinectCloud::computeNormals() {
	//~ 
	//~ std::vector<VCGVertex> vertexList;
	//~ VCGVertex vcgVertex;
	//~ 
	//~ for (int i = 0; i < cloud.size(); i++) {
		//~ 
		//~ vcgVertex.P() = vcg::Point3d(cloud[i][0], cloud[i][1], cloud[i][2]);
		//~ vertexList.push_back(vcgVertex);
	//~ }
	//~ 
	//~ vcg::NormalExtrapolation<std::vector<VCGVertex> >::ExtrapolateNormals(vertexList.begin(), vertexList.end(), 5, -1, vcg::NormalExtrapolation<std::vector<VCGVertex> >::IsCorrect);	
		//~ 
	//~ for (int i = 0; i < normals.size(); i++) {
		//~ normals[i][0] = vertexList[i].N()[0];
		//~ normals[i][1] = vertexList[i].N()[1];
		//~ normals[i][2] = vertexList[i].N()[2];
	//~ }
	//~ 
	//~ reconstructionSteps[4] = true;
//~ }

std::vector< std::vector<float> > KinectCloud::getCloud() {
	return cloud;
}

std::vector< std::vector<int> > KinectCloud::getCloudRgb() {
	return cloudRgb;
}

std::vector< std::vector<int> > KinectCloud::getCloudRgbMapping() {
	return cloudRgbMapping;
}

//~ std::vector< std::vector<float> > KinectCloud::getNormals() {
	//~ return normals;
//~ }

std::vector<bool> KinectCloud::getValidValues() {
	return validValues;
}

std::vector<bool> KinectCloud::getReconstructionSteps() {
	return reconstructionSteps;
}
