/* This file is part of the Kiretu, a Kinect reconstruction tutor.
 * 
 * http://pille.iwr.uni-heidelberg.de/~kinect01/
 * 
 * The code ist licensed to you under the terms of the GNU General Public 
 * License, version 2.0. See the GPL2 file for the text of the license or the 
 * following URL:
 * 
 * http://www.gnu.org/licenses/gpl-2.0.txt
 * 
 * If you redistribute this file in source form, modified or unmodified, you
 * have to leave this header intact and distribute it under the same terms with 
 * the GPL2 file.
 * 
 * Binary distributions must follow the binary distribution requirements of
 * either License.
 * 
 */
 
 /** \file Util.cpp
 * 
 * Implementation of Util.h. Contains some often used functions.
 * 
 * \author		Daniel Wunderlich (d.wunderlich@stud.uni-heidelberg.de)
 * \version		0.5
 * \date		2011-01-05
 */

#include "Util.h"

// Convertion utilities

float strToFloat(std::string &str) {
	
	float number;
	
	std::stringstream strstream;
	
	strstream << str;
	strstream >> number;
	
	return number;
}

std::string intToStr(int i) {
	
	std::string str;
	std::stringstream strstream;
	
	strstream << i;
	
	str = strstream.str();
	
	return str;
}

// Vector/Matrix utilities

void printVec(std::vector<float> &vec) {
	
	for (std::vector<float>::iterator it = vec.begin(); it < vec.end(); it++) {
		std::cout << *it << "\t";
	}
	std::cout << std::endl << std::endl;
}

void printMat(std::vector< std::vector<float> > &mat) {
	
	for (std::vector< std::vector<float> >::iterator vec = mat.begin(); vec < mat.end(); vec++) {
		for (std::vector<float>::iterator it = (*vec).begin(); it < (*vec).end(); it++) {
			std::cout << *it << "\t";
		}
		std::cout << std::endl;
	}
	std::cout << std::endl << std::endl;
}

void matMult(std::vector< std::vector<float> > &mat, std::vector<float> &vec, std::vector<float> &res) {
	
	for (int row = 0; row < mat.size(); row++) {
		
		res[row] = 0;
		for (int col = 0; col < mat[row].size(); col++) {
			res[row] += mat[row][col] * vec[col];
		}
	}
}

void vecAdd(std::vector<float> &vec1, std::vector<float> &vec2, std::vector<float> &res) {
	
	for (int i = 0; i < vec1.size(); i++) {
		res[i] = vec1[i] + vec2[i];
	}
}
