/* This file is part of the Kiretu, a Kinect reconstruction tutor.
 * 
 * http://pille.iwr.uni-heidelberg.de/~kinect01/
 * 
 * The code ist licensed to you under the terms of the GNU General Public 
 * License, version 2.0. See the GPL2 file for the text of the license or the 
 * following URL:
 * 
 * http://www.gnu.org/licenses/gpl-2.0.txt
 * 
 * If you redistribute this file in source form, modified or unmodified, you
 * have to leave this header intact and distribute it under the same terms with 
 * the GPL2 file.
 * 
 * Binary distributions must follow the binary distribution requirements of
 * the License.
 * 
 */
 
 /** \file Util.h
 * 
 * Util.h contains some often used functions.
 * 
 * \author		Daniel Wunderlich (d.wunderlich@stud.uni-heidelberg.de)
 * \version		0.8
 * \date		2011-01-26
 */

#include <vector>
#include <iostream>
#include <sstream>
#include <string>

// Conversion utilities

/**	\brief Converts a string to float.
 * 	
 * 	\param 	str		The string, which should be converted.
 * 
 * 	\return The converted float.
 */ 
float strToFloat(std::string &str);

std::string intToStr(int i);


// Vector/Matrix utilities

/**	\brief Prints the given vector to the standard-output.
 * 	
 * 	\param 	vec	The vector, which should be printed.
 */ 
void printVec(std::vector<float> &vec);


/**	\brief Prints the given matrix to the standard-output.
 * 	
 * 	\param 	mat	The matrix, which should be printed.
 */ 
void printMat(std::vector< std::vector<float> > &mat);


/**	\brief 	Multiplies the matrix mat with the vector vec and saves the result in 
 * 			the vector res.
 * 	
 * 	\param 	mat	The matrix, which should be multiplied.
 * 	\param 	vec	The vector, which should be multiplied.
 * 	\param 	res	The vector the result should be saved in.
 */ 
void matMult(std::vector< std::vector<float> > &mat, std::vector<float> &vec, std::vector<float> &res);


/**	\brief 	Adds the vectores vec1 and vec2 and saves the result in the vector 
 * 			res.
 * 	
 * 	\param 	vec1	The first vector, which should be added.
 * 	\param 	vec2	The second vector, which should be added.
 * 	\param 	res		The vector the result should be saved in.
 */ 
void vecAdd(std::vector<float> &vec1, std::vector<float> &vec2, std::vector<float> &res);
